<?php

namespace jrosset\ExecutionTimer;

/**
 * Store a time duration
 */
class Duration {
    const EPSILON = 0.0000001;

    /**
     * @var float The time duration
     */
    private $duration;

    /**
     * Duration constructor.
     *
     * @param float $duration Time duration, in seconds fraction
     */
    public function __construct ($duration = 0) {
        $this->duration = abs($duration);
    }

    /**
     * Convert duration to string
     *
     * @return string The duration, as string
     */
    public function __toString () {
        return $this->getDays()->asRelativeWhole() . ' '
               . self::showUnit($this->getHours()) . ':'
               . self::showUnit($this->getMinutes()) . ':'
               . self::showUnit($this->getSeconds()) . '.'
               . self::showUnit($this->getMilliseconds());
    }
    private static function showUnit (DurationUnit $durationUnit) {
        $max = $durationUnit->getUnitMax();
        $length = mb_strlen($max);
        if (preg_match('/^1[0]*$/', $max)) {
            $length -= 1;
        }

        return str_pad($durationUnit->asRelativeWhole(), $length, '0', STR_PAD_LEFT);
    }
    /**
     * Convert duration to user friendly string
     *
     * @return string The duration, as user friendly string
     */
    public function toUserString () {
        $parts = array(
            $this->getDays()->asRelativeWhole() > 0 ? $this->getDays()->asRelativeWhole() . ' days' : '',
            $this->getHours()->asRelativeWhole() > 0 ? $this->getHours()->asRelativeWhole() . ' hours' : '',
            $this->getMinutes()->asRelativeWhole() > 0 ? $this->getMinutes()->asRelativeWhole() . ' minutes' : '',
            $this->getSeconds()->asRelativeWhole() > 0 ? $this->getSeconds()->asRelativeWhole() . ' seconds' : '',
            $this->getMilliseconds()->asRelativeWhole() > 0 ? $this->getMilliseconds()->asRelativeWhole() . ' milliseconds' : '',
        );

        $parts = array_filter(
            $parts,
            function ($part) {
                return $part != '';
            }
        );
        $last = array_pop($parts);

        return (count($parts) > 0 ? implode(' ', $parts) . ' and ' : '') . $last;
    }

    /**
     * The base time duration
     *
     * @return float The base time duration
     */
    public function getDuration () {
        return $this->duration;
    }
    /**
     * Contains a time duration ? (more than 0 microseconds)
     *
     * @return bool True if contains a time duration
     */
    public function hasTime () {
        return abs($this->duration - 0) > self::EPSILON;
    }

    /**
     * The time in microseconds
     *
     * @return DurationUnit The time in microseconds
     */
    public function getMicroseconds () {
        return new DurationUnit($this->duration * 1000000, DurationUnit::UNIT_MICROSECONDS);        // 1000 milliseconds * 1000 microseconds
    }
    /**
     * The time in milliseconds
     *
     * @return DurationUnit The time in milliseconds
     */
    public function getMilliseconds () {
        return new DurationUnit($this->duration * 1000, DurationUnit::UNIT_MILLISECONDS);           // 1000 milliseconds
    }
    /**
     * The time in seconds
     *
     * @return DurationUnit The time in seconds
     */
    public function getSeconds () {
        return new DurationUnit($this->duration, DurationUnit::UNIT_SECONDS);
    }
    /**
     * The time in seconds
     *
     * @return DurationUnit The time in seconds
     */
    public function getMinutes () {
        return new DurationUnit($this->duration / 60, DurationUnit::UNIT_MINUTES);          // 60 seconds
    }
    /**
     * The time in hours
     *
     * @return DurationUnit The time in hours
     */
    public function getHours () {
        return new DurationUnit($this->duration / 3600, DurationUnit::UNIT_HOURS);          // 60 seconds * 60 minutes
    }
    /**
     * The time in days
     *
     * @return DurationUnit The time in days
     */
    public function getDays() {
        return new DurationUnit($this->duration / 86400, DurationUnit::UNIT_DAYS);          // 60 seconds * 60 minutes * 24 hours
    }
}