<?php

namespace jrosset\ExecutionTimer;

/**
 * The time measurer
 *
 * The main class
 */
class ExecutionTimer {
    /**
     * @var float Initial start time, in fractional seconds
     */
    private $timeStart;
    /**
     * @var float[] Stop time of steps, in fractional seconds
     */
    private $timeSteps;
    /**
     * @var float Stop time, in fractional seconds
     */
    private $timeStop;

    /**
     * New time measurer
     *
     * @param bool $autostart Start time measurement ?
     */
    public function __construct ($autostart = true) {
        $this->timeStart = 0;
        $this->timeSteps = array();
        $this->timeStop = 0;

        if ($autostart) {
            $this->start();
        }
    }

    /**
     * Measure current time as seconds with fractions
     *
     * @return float The current time
     */
    private function measureTime () {
        return microtime(true);
    }

    /**
     * Start a new time measurement
     */
    public function start () {
        $this->timeStart = $this->measureTime();
        $this->timeSteps = array();
        $this->timeStop = 0;
    }
    /**
     * Measure a new step
     *
     * @param string name The name of the step
     */
    public function step ($name = '') {
        $this->timeSteps[$name === '' ? count($this->timeSteps) + 1 : $name] = $this->measureTime();
    }
    /**
     * Stop time measurement
     */
    public function stop () {
        $this->timeStop = $this->measureTime();
    }

    /**
     * Is time measured ?
     *
     * @return bool True if time is measured
     */
    public function isStart () {
        return $this->timeStop === 0;
    }

    /**
     * The number of steps measured
     *
     * @return int The number of steps measured
     */
    public function nbSteps () {
        return count($this->timeSteps);
    }
    /**
     * Has at least one step measured ?
     *
     * @return bool True if a least on step measured
     */
    public function hasSteps () {
        return $this->nbSteps() > 0;
    }
    /**
     * List of steps durations
     *
     * Each step provide an array of {@see Duration} with two cells :
     *  # The duration relative to previous step or initial start time if first step
     *  # The duration (relative to initial start time)
     *
     * @return Duration[][] List of steps durations
     */
    public function getStepsDuration () {
        $steps = array();
        if (!$this->hasSteps()) {
            return $steps;
        }

        $timePrevious = $this->timeStart;
        foreach ($this->timeSteps as $stepName => $timeStep) {
            $steps[$stepName] = array(new Duration($timeStep - $timePrevious), new Duration($timeStep - $this->timeStart));
            $timePrevious = $timeStep;
        }

        return $steps;
    }

    /**
     * The last step duration relative to previous step or initial start time if first step
     *
     * @return Duration The time duration
     */
    public function getLastStepRelativeDuration () {
        switch($this->nbSteps()) {
            case 0:
                return new Duration();

            case 1:
                return new Duration(reset($this->timeSteps) - $this->timeStart);

            default:
                $time = end($this->timeSteps);
                return new Duration( $time- prev($this->timeSteps));
        }
    }
    /**
     * The last step duration (relative to initial start time)
     *
     * @return Duration The time duration
     */
    public function getLastStepDuration () {
        if($this->nbSteps() === 0) {
            return new Duration();
        }

        return new Duration( end($this->timeSteps) - $this->timeStart);
    }

    /**
     * The total duration relative to previous step or initial start time if first step
     *
     * @return Duration The time duration
     */
    public function getRelativeDuration () {
        if($this->nbSteps() === 0) {
            return $this->getDuration();
        }

        return new Duration( $this->timeStop - end($this->timeSteps));
    }
    /**
     * The total duration (relative to initial start time)
     *
     * @return Duration The time duration
     */
    public function getDuration () {
        return new Duration($this->timeStop - $this->timeStart);
    }
}